
-------------------------------------------------------------------------------
-- Optimization
-------------------------------------------------------------------------------

-- GLOBALS: _G, CHAT_FRAMES, COMBATLOG, next

local format = format
local type = type

-------------------------------------------------------------------------------
-- Namespace
-------------------------------------------------------------------------------

local A, B, C, D = unpack(select(2, ...)) -- AddOn, Border, Constants, Database
local M = A:NewModule("Chat Abbreviations")

local CHANNEL_TO_ABBREVIATION = {
	GUILD = "G",
	OFFICER = "O",
	INSTANCE_CHAT = "I",
	INSTANCE_CHAT_LEADER = "IL",
	PARTY = "P",
	PARTY_GUIDE = "PL",
	PARTY_LEADER = "PL",
	RAID = "R",
	RAID_LEADER = "RL",
}

local MATCH_TIMESTAMP = "^(%d-:%d-[:%d AMP]+) "
local MATCH_WHISPER_TO = "^" .. CHAT_WHISPER_INFORM_GET:format("(.-|h)")
local MATCH_WHISPER_FROM = "^" .. CHAT_WHISPER_GET:format("(.-|h)")
local MATCH_SAY = "^" .. CHAT_SAY_GET:format("(.-|h)")
local MATCH_YELL = "^" .. CHAT_YELL_GET:format("(.-|h)")
local MATCH_RAID_WARNING = "^" .. CHAT_RAID_WARNING_GET:format("(.-|h)")
local MATCH_FLAG_AFK = CHAT_FLAG_AFK .. "(|Hplayer)"
local MATCH_FLAG_DND = CHAT_FLAG_DND .. "(|Hplayer)"

-------------------------------------------------------------------------------
-- Helpers
-------------------------------------------------------------------------------

local function AbbreviateChannel(channel)
	return format("|Hchannel:%s|h[%s]|h", channel, CHANNEL_TO_ABBREVIATION[channel] or channel:gsub("^.-:", ""))
end

local function AbbreviateMessage(text, ...)
	local time = text:match(MATCH_TIMESTAMP)

	-- Remove the timestamp.
	text = text:gsub(MATCH_TIMESTAMP, "")

	-- Automatically scale the size of textures.
	text = text:gsub("|T([^:]-):%d+|t", "|T%1:0|t")
	text = text:gsub("|T([^:]-):%d+:%d+(.-)|t", "|T%1:0:0%2|t")
	text = text:gsub("|A:([^:]-):.-|a", "|A:%1:0:0|a")

	-- Abbreviate message descriptors.
	text = text:gsub(MATCH_WHISPER_TO, "[W To] %1 ")
	text = text:gsub(MATCH_WHISPER_FROM, "[W From] %1 ")
	text = text:gsub(MATCH_SAY, "[S] %1 ")
	text = text:gsub(MATCH_YELL, "[Y] %1 ")
	text = text:gsub(MATCH_RAID_WARNING, "[RW] %1 ")

	-- Abbreviate channel names.
	text = text:gsub("^|Hchannel:(.-)|h.-|h", AbbreviateChannel)

	-- Strip the <Away> and <Busy> prefix from player names.
	text = text:gsub(MATCH_FLAG_AFK, "%1")
	text = text:gsub(MATCH_FLAG_DND, "%1")

	-- Strip the colon symbol from messages without a player name.
	text = text:gsub("^(|Hchannel.-|h.-|h) :", "%1")

	-- Restore the timestamp.
	if time then
		text = "[" .. time .. "] " .. text
	end

	return text, ...
end

-------------------------------------------------------------------------------
-- Module
-------------------------------------------------------------------------------

function M:OnEnable()
	self:ConfigureChatFrames()
end

function M:ConfigureChatFrames()
	for _, frameName in next, CHAT_FRAMES do
		self:ConfigureChatFrame(_G[frameName])
	end
end

function M:ConfigureChatFrame(chatFrame)
	if chatFrame ~= COMBATLOG and not self:IsHooked(chatFrame, "AddMessage") then
		self:SecureHook(chatFrame, "AddMessage", "AbbreviateMessageByText")
		self:AbbreviateAllMessages(chatFrame)
	end
end

function M:AbbreviateAllMessages(chatFrame)
	chatFrame:TransformMessages(function (text) return type(text) == "string" end, AbbreviateMessage)
end

function M:AbbreviateMessageByText(chatFrame, text1)
	if type(text1) == "string" then
		chatFrame:TransformMessages(function (text2) return text1 == text2 end, AbbreviateMessage)
	end
end
